/**
 * \file LowConventionalTaw.cpp
 * \author Christopher Ruwisch (christopher.ruwisch@tu-berlin.de)
 * \brief
 * \version 0.1
 * \date 2023-10-30
 *
 * @copyright Copyright (c) 2023
 *
 */

#include "low_t_tail_empennage_design.h"

#include <CGAL/Surface_mesh/IO/PLY.h>
#include <aircraftGeometry2/geometry/factory.h>
#include <aircraftGeometry2/io/convert.h>
#include <aircraftGeometry2/io/dat.h>
#include <aircraftGeometry2/processing/measure.h>
#include <aircraftGeometry2/processing/transform.h>

#include <format>
#include <numbers>

#include "t_tail_empennage_design_config.h"
#include "t_tail_empennage_design_data.h"
#include "lib/design_methods/volumeCoefficientMethod.h"
#include "lib/mass_methods/flopsMassMethod.h"

static std::string to_filename(std::string s) {
  std::replace(s.begin(), s.end(), ' ', '_');
  return s;
}

namespace taw {
namespace low {

T_Tail::T_Tail(const std::shared_ptr<RuntimeIO> &rtIO)
    : rtIO(rtIO),
      config(std::make_shared<taw::low::T_Tail_Config>()),
      data(std::make_shared<taw::low::T_Tail_Data>()),
      design_mode_runner({{"mode_0", [this]() { design(); }}, {"mode_1", [this]() { redesign(); }}}),
      mass_mode_runner({{"mode_0", [this]() { flops_mass(); }}, {"mode_1", [this]() { raymer_mass(); }}}) {};
      //reporter(rtIO) {};

void T_Tail::initialize() {
  // Setup methods for mode
  myRuntimeInfo->out << "Initializing empennage" << std::endl;
  myRuntimeInfo->out << "Empennage type -> T-Tail (vertical_stabilizer/horizontal_stabilizer)" << std::endl;

  /* Read configuration data */
  config->read(rtIO->moduleConfig);

  /* Create airfoils directory in project path (only if non-existing -> e.g. if standalone usage)*/
  rtIO->createAirfoilDataDir();

  /* Read acxml data - non aircraftgeometry2 */
  data->read(rtIO->acxml);

  /* Read acxml data - aircraftgeometry2 */
  data->read_wing(rtIO->acxmlAccess, rtIO->getAirfoilDataDir());
  data->read_fuselage(rtIO->acxmlAccess, rtIO->getGeometryDir());

  /* Select modes */
  selected_design_mode = config->design_mode.value();
  selected_mass_mode = config->mass_mode.value();

  /* Create airfoils library */
  airfoils_library =
      std::make_shared<Airfoils>(static_cast<std::filesystem::path>(config->common_airfoil_data_path.value()));

  /* Append runtimeIO airfoils data directory */
  airfoils_library->add_directory_airfoils(rtIO->getAirfoilDataDir());


}

void T_Tail::run() {
  /* Run methods */
  myRuntimeInfo->out << "Running empennage -> T-tail" << std::endl;
  /* Run design method */
  design_mode_runner[selected_design_mode]();

  /* Run mass method */
  mass_mode_runner[selected_mass_mode]();
}

void T_Tail::update() {
  myRuntimeInfo->out << "Updating empennage -> T-tail" << std::endl;
  /* Update aircraft exchange file */
  data->update(rtIO->acxml);
}

void T_Tail::report() {
  /* Create reports */
  myRuntimeInfo->out << "Reporting empennage -> T-tail" << std::endl;

  /* Set html report data and plots */
  //set_html_body();

  /* Generate reports */
  //reporter.generateReports();

  /* Create STL file (only for debugging reasons) */
  data->vertical_stabilizer.origin =
      geom2::Point_3(data->empennage_position.x.value() + data->vertical_stabilizer.origin.x(),
                     data->empennage_position.y.value() + data->vertical_stabilizer.origin.y(),
                     data->empennage_position.z.value() + data->vertical_stabilizer.origin.z());
  geom2::Mesh vertical_stabilizer_mesh = geom2::transform::to_mesh(data->vertical_stabilizer);
  /* Create STL file (vertical stabilizer) */
  CGAL::IO::write_STL(to_filename(data->vertical_stabilizer.name) + ".stl", vertical_stabilizer_mesh);
  data->horizontal_stabilizer.origin =
      geom2::Point_3(data->empennage_position.x.value() + data->horizontal_stabilizer.origin.x(),
                     data->empennage_position.y.value() + data->horizontal_stabilizer.origin.y(),
                     data->empennage_position.z.value() + data->horizontal_stabilizer.origin.z());
  geom2::Mesh horizontal_stabilizer_mesh = geom2::transform::to_mesh(data->horizontal_stabilizer);

  /* Create STL File (horizontal stabilizer (one sided)) */
  CGAL::IO::write_STL(to_filename(data->horizontal_stabilizer.name) + ".stl", horizontal_stabilizer_mesh);
}

void T_Tail::save() {
  // save data if necessary
}

void T_Tail::design() {
  myRuntimeInfo->out << "Mode active -> Design" << std::endl;

  double wing_reference_area = geom2::measure::reference_area(data->wing);
  double wing_span = geom2::measure::span(data->wing);
  double wing_sweep = fabs(geom2::measure::sweep(data->wing, -0.25 * wing_span, 0.25));
  double wing_taper_ratio = geom2::measure::taper_ratio(data->wing);
  double wing_aspect_ratio = geom2::measure::aspect_ratio(data->wing);
  double span_position_mac = geom2::measure::mean_aerodynamic_chord_position(data->wing);
  double wing_mac = geom2::measure::chord(data->wing, span_position_mac);
  geom2::Point_3 wing_neutral_position = geom2::Point_3(
      data->wing_position_x.value() + geom2::measure::offset_LE(data->wing, span_position_mac).x() + 0.25 * wing_mac,
      0., 0.);
  double fuselage_length = geom2::measure::length(data->fuselage);

  // Vertical stabilizer (vs) computation - data from configuration
  design_vertical_stabilizer(wing_sweep, wing_reference_area, wing_span, wing_taper_ratio, wing_aspect_ratio,
                             wing_neutral_position, fuselage_length);

  for (auto device : config->vertical_stabilizer.control_devices) {
    data->vertical_stabilizer_control_devices.push_back(control_device(
        device.type.value(), device.position.inner_position_chord_from.value(),
        device.position.inner_position_chord_to.value(), device.position.inner_position_spanwise.value(),
        device.position.outer_position_chord_from.value(), device.position.outer_position_chord_to.value(),
        device.position.outer_position_spanwise.value()));
    data->vertical_stabilizer_control_devices_deflections.push_back(
        {device.deflection_full_negative.value(), device.deflection_full_positive.value()});
  }
  for (auto spar_element : config->vertical_stabilizer.spars) {
    data->vertical_stabilizer_spars.push_back(spar(
        spar_element.name.value(), spar_element.position.inner_position_chord_from.value(),
        spar_element.position.inner_position_chord_to.value(), spar_element.position.inner_position_spanwise.value(),
        spar_element.position.outer_position_chord_from.value(), spar_element.position.outer_position_chord_to.value(),
        spar_element.position.outer_position_spanwise.value()));
  }

  // Horizontal stabilizer (hs) computation - data from configuration
  design_horizontal_stabilizer(wing_sweep, wing_reference_area, wing_mac, wing_taper_ratio, wing_aspect_ratio,
                               wing_neutral_position, fuselage_length);
  for (auto device : config->horizontal_stabilizer.control_devices) {
    data->horizontal_stabilizer_control_devices.push_back(control_device(
        device.type.value(), device.position.inner_position_chord_from.value(),
        device.position.inner_position_chord_to.value(), device.position.inner_position_spanwise.value(),
        device.position.outer_position_chord_from.value(), device.position.outer_position_chord_to.value(),
        device.position.outer_position_spanwise.value()));
    data->horizontal_stabilizer_control_devices_deflections.push_back(
        {device.deflection_full_negative.value(), device.deflection_full_positive.value()});
  }
  for (auto spar_element : config->horizontal_stabilizer.spars) {
    data->horizontal_stabilizer_spars.push_back(spar(
        spar_element.name.value(), spar_element.position.inner_position_chord_from.value(),
        spar_element.position.inner_position_chord_to.value(), spar_element.position.inner_position_spanwise.value(),
        spar_element.position.outer_position_chord_from.value(), spar_element.position.outer_position_chord_to.value(),
        spar_element.position.outer_position_spanwise.value()));
  }

  /* Set empennage position -> vertical and horizontal stabilizer */
  set_empennage_position();
}

void T_Tail::redesign() {
  myRuntimeInfo->out << "Redesign mode active Conventional Tail" << std::endl;
  double wing_sweep = 0.0;                                               // data...;
  double wing_reference_area = 0.0;                                      // data...;
  double wing_span = 0.0;                                                // data...;
  double wing_mac = 0.0;                                                 // data...;
  double wing_taper_ratio = 0.0;                                         // data...;
  double wing_aspect_ratio = 0.0;                                        // data...;
  geom2::Point_3 wing_neutral_position = geom2::Point_3(0.0, 0.0, 0.0);  //...;
  double fuselage_length = 0.0;                                          // data...;
}

void T_Tail::design_vertical_stabilizer(const double wing_sweep, const double wing_reference_area,
                                              const double wing_span, const double wing_taper_ratio,
                                              const double wing_aspect_ratio, geom2::Point_3 &wing_neutral_position,
                                              const double fuselage_length) {
  geom2::Direction_3 orientation = geom2::Direction_3(0.0, 0.0, -1.0);
  data->vertical_stabilizer =
      design_stabilizer(wing_sweep, wing_reference_area, wing_span, wing_taper_ratio, wing_aspect_ratio,
                        wing_neutral_position, fuselage_length, config->vertical_stabilizer, orientation, false);
  data->vertical_stabilizer.name = config->vertical_stabilizer.name.value();
  data->vertical_stabilizer.sections.at(0).name = config->vertical_stabilizer.inner_profile.value();
  data->vertical_stabilizer.sections.at(1).name = config->vertical_stabilizer.outer_profile.value();
  data->vertical_stabilizer.is_symmetric = false;
}

void T_Tail::design_horizontal_stabilizer(const double wing_sweep, const double wing_reference_area,
                                                const double wing_mac, const double wing_taper_ratio,
                                                const double wing_aspect_ratio, geom2::Point_3 &wing_neutral_position,
                                                const double fuselage_length) {
  geom2::Direction_3 orientation = geom2::Direction_3(0.0, -1.0, 0.0);
  double vertical_stabilizer_tip_height = geom2::measure::span(data->vertical_stabilizer);
  geom2::Point_3 vertical_stabilizer_leading_edge_position_tip = geom2::measure::offset_LE(data->vertical_stabilizer,-vertical_stabilizer_tip_height);
  double vertical_stabilizer_chord_chord_at_tip = geom2::measure::chord(data->vertical_stabilizer,-vertical_stabilizer_tip_height);
  double reference_length_for_T_tail = data->vertical_stabilizer.origin.x() + vertical_stabilizer_leading_edge_position_tip.x() + vertical_stabilizer_chord_chord_at_tip;

  data->horizontal_stabilizer =
      design_stabilizer(wing_sweep, wing_reference_area, wing_mac, wing_taper_ratio, wing_aspect_ratio,
                        wing_neutral_position, reference_length_for_T_tail, config->horizontal_stabilizer, orientation, true);
  //data->horizontal_stabilizer.origin = geom2::Point_3(data->horizontal_stabilizer.origin.x(), data->horizontal_stabilizer.origin.y(), data->horizontal_stabilizer.origin.z() - vertical_stabilizer_tip_height);
  data->horizontal_stabilizer.name = config->horizontal_stabilizer.name.value();
  data->horizontal_stabilizer.sections.at(0).name = config->horizontal_stabilizer.inner_profile.value();
  data->horizontal_stabilizer.sections.at(1).name = config->horizontal_stabilizer.outer_profile.value();
  data->horizontal_stabilizer.is_symmetric = true;
}

void T_Tail::redesign_vertical_stabilizer(const double wing_sweep, const double wing_reference_area,
                                                const double wing_span, geom2::Point_3 &wing_neutral_position,
                                                const double fuselage_length) {
  // const double aspect_ratio = geom2::measure:: ...
  // const double taper_ratio = geom2::measure:: ...
  // geom2::Polygon_2 profile_root = factory.sections.front().shape
  // geom2::Polygon_2 profile_tip = factory.sections.back().shape
}

void T_Tail::redesign_horizontal_stabilizer(const double wing_sweep, const double wing_reference_area,
                                                  const double wing_span, geom2::Point_3 &wing_neutral_position,
                                                  const double fuselage_length) {
  // const double aspect_ratio = geom2::measure:: ...
  // const double taper_ratio = geom2::measure:: ...
  // geom2::Polygon_2 profile_root = factory.sections.front().shape
  // geom2::Polygon_2 profile_tip = factory.sections.back().shape
}

geom2::MultisectionSurface<geom2::AirfoilSection> T_Tail::design_stabilizer(
    const double wing_sweep_rad, const double wing_reference_area, const double reference_length,
    geom2::Point_3 &wing_neutral_position, const double fuselage_length, geom2::Direction_3 &orientation,
    const double volume_coefficient, const double delta_sweep_rad, const double stabilizer_taper_ratio,
    const double stabilizer_aspect_ratio, geom2::Point_3 &offset_position, const std::string &inner_profile_name,
    const std::string &outer_profile_name, bool symmetric, int number_of_stabilizers) {
  /* Read inner profile */
  geom2::AirfoilSection profile_root(airfoils_library->get_airfoil(inner_profile_name));
  airfoils_library->copy_available_airfoil(inner_profile_name, rtIO->getAirfoilDataDir());
  profile_root.name = inner_profile_name;

  /* Read outer profile */
  geom2::AirfoilSection profile_tip(airfoils_library->get_airfoil(inner_profile_name));
  airfoils_library->copy_available_airfoil(inner_profile_name, rtIO->getAirfoilDataDir());
  profile_tip.name = outer_profile_name;

  /* Create volume coefficient method */
  VolumeCoefficientMethod volume_coefficient_method;

  /* Compute geometry by volume coefficient method */
  volume_coefficient_method.compute_geometry(
      number_of_stabilizers, volume_coefficient, wing_sweep_rad + delta_sweep_rad, stabilizer_taper_ratio,
      stabilizer_aspect_ratio, wing_reference_area, reference_length, fuselage_length, wing_neutral_position,
      offset_position, orientation, symmetric);

  /* Set dihedral to 0 */
  double dihedral = 0.0;
  /* check if not direction is in horizontal axis */
  if (orientation.dy() < -0.5) {
    dihedral = geom2::measure::dihedral(data->wing, -0.25 * geom2::measure::span(data->wing));
  }
  return volume_coefficient_method.get_design(profile_root, profile_tip, dihedral);
}

geom2::MultisectionSurface<geom2::AirfoilSection> T_Tail::design_stabilizer(
    const double wing_sweep_rad, const double wing_reference_area, const double reference_length,
    const double wing_taper_ratio, const double wing_aspect_ratio, geom2::Point_3 &wing_neutral_position,
    const double fuselage_length, TailElement &tail, geom2::Direction_3 &orientation, bool symmetric) {
  geom2::Point_3 offset_position(tail.rear_x_offset.value(), tail.centerline_y_offset.value(),
                                 tail.centerline_z_offset.value());
  double volume_coefficient = tail.volume_coefficient.value();

  /* Set values if volume_coefficient is set to small */
  if (fabs(volume_coefficient) < 1E-4) {
    /* Compute values dependend on orientation - threshold at 0.95 -> empirical */
    if (std::fabs(orientation.dz()) > 0.95) {
      /* Vertical stabilizer - Raymer -> P.112 Table 6.4 */
      volume_coefficient = 0.0855;
    } else {
      /* Horizontal stabilizer - Raymer -> P.112 Table 6.4 */
      volume_coefficient = 0.95;
    }
  }
  return design_stabilizer(wing_sweep_rad, wing_reference_area, reference_length, wing_neutral_position,
                           fuselage_length, orientation, volume_coefficient, tail.delta_sweep.value(),
                           wing_taper_ratio * tail.factor_taper_ratio.value(),
                           wing_aspect_ratio * tail.factor_aspect_ratio.value(), offset_position,
                           tail.inner_profile.value(), tail.outer_profile.value(), symmetric, 1);
}

void T_Tail::set_empennage_position(void) {
  // Vertical stabilizer and horizontal stabilizer set both to equal vertical position

  data->horizontal_stabilizer.origin =
      geom2::Point_3(data->horizontal_stabilizer.origin.x(), data->horizontal_stabilizer.origin.y(),
                     -data->vertical_stabilizer.sections.back().origin.z());

  data->vertical_stabilizer.origin =
      geom2::Point_3(data->vertical_stabilizer.origin.x(), data->vertical_stabilizer.origin.y(),
                     data->vertical_stabilizer.origin.z() );
  if (data->vertical_stabilizer.origin.x() < data->horizontal_stabilizer.origin.x()) {
    data->empennage_position.set_xyz(data->vertical_stabilizer.origin.x(), data->vertical_stabilizer.origin.y(),
                                     data->vertical_stabilizer.origin.z());
    data->horizontal_stabilizer.origin =
        geom2::Point_3(data->horizontal_stabilizer.origin.x() - data->empennage_position.x.value(),
                       data->horizontal_stabilizer.origin.y() - data->empennage_position.y.value(),
                       data->horizontal_stabilizer.origin.z() - data->empennage_position.z.value());
    data->vertical_stabilizer.origin = geom2::Point_3(0.0, 0.0, 0.0);
  } else {
    data->empennage_position.set_xyz(data->horizontal_stabilizer.origin.x(), data->horizontal_stabilizer.origin.y(),
                                     data->horizontal_stabilizer.origin.z());
    data->vertical_stabilizer.origin =
        geom2::Point_3(data->vertical_stabilizer.origin.x() - data->empennage_position.x.value(),
                       data->vertical_stabilizer.origin.y() - data->empennage_position.y.value(),
                       data->vertical_stabilizer.origin.z() - data->empennage_position.z.value());
    data->horizontal_stabilizer.origin = geom2::Point_3(0.0, 0.0, 0.0);
  }

  myRuntimeInfo->out << "Empennage position (x y z) -> " << data->empennage_position.xyz() << std::endl;
}

// == Spars and Controls ==============================================================================================

geom2::MultisectionSurface<geom2::PolygonSection> T_Tail::control_device(
    const std::string &name, double inner_chord_rel_pos_from, double inner_chord_rel_pos_to,
    double inner_spanwise_rel_pos, double outer_chord_rel_pos_from, double outer_chord_rel_pos_to,
    double outer_spanwise_rel_pos) {
  geom2::MultisectionSurface<geom2::PolygonSection> control_surface;
  geom2::Polygon_2 inner_chord;
  inner_chord.push_back(geom2::Point_2(inner_chord_rel_pos_from, 0.0));
  inner_chord.push_back(geom2::Point_2(inner_chord_rel_pos_to, 0.0));

  geom2::Polygon_2 outer_chord;
  outer_chord.push_back(geom2::Point_2(outer_chord_rel_pos_from, 0.0));
  outer_chord.push_back(geom2::Point_2(outer_chord_rel_pos_to, 0.0));

  control_surface.sections.emplace_back(inner_chord);
  control_surface.sections.emplace_back(outer_chord);

  control_surface.sections.front().origin = geom2::Point_3(0.0, 0.0, inner_spanwise_rel_pos);
  control_surface.sections.back().origin = geom2::Point_3(0.0, 0.0, outer_spanwise_rel_pos);

  control_surface.name = name;
  return control_surface;
}

geom2::MultisectionSurface<geom2::PolygonSection> T_Tail::spar(
    const std::string &name, double inner_chord_rel_pos_from, double inner_chord_rel_pos_to,
    double inner_spanwise_rel_pos, double outer_chord_rel_pos_from, double outer_chord_rel_pos_to,
    double outer_spanwise_rel_pos) {
  geom2::MultisectionSurface<geom2::PolygonSection> spar;
  geom2::Polygon_2 inner_chord;
  inner_chord.push_back(geom2::Point_2(inner_chord_rel_pos_from, 0.0));
  inner_chord.push_back(geom2::Point_2(inner_chord_rel_pos_to, 0.0));

  geom2::Polygon_2 outer_chord;
  outer_chord.push_back(geom2::Point_2(outer_chord_rel_pos_from, 0.0));
  outer_chord.push_back(geom2::Point_2(outer_chord_rel_pos_to, 0.0));

  spar.sections.emplace_back(inner_chord);
  spar.sections.emplace_back(outer_chord);

  spar.sections.front().origin = geom2::Point_3(0.0, 0.0, inner_spanwise_rel_pos);
  spar.sections.back().origin = geom2::Point_3(0.0, 0.0, outer_spanwise_rel_pos);
  spar.name = name;

  return spar;
}

// == Mass ============================================================================================================
void T_Tail::flops_mass() {
  myRuntimeInfo->out << "Mode active -> Mass computation - FLOPS" << std::endl;
  const double maximum_takeoff_mass = data->maximum_takeoff_mass.value();

  data->vertical_stabilizer_mass.data["mass"] =
      flops::vertical_stabilizer_mass(1, geom2::measure::reference_area(data->vertical_stabilizer),
                                      geom2::measure::taper_ratio(data->vertical_stabilizer), maximum_takeoff_mass);
  myRuntimeInfo->out << std::format("Vertical stabilizer mass ... {}",
                                    data->vertical_stabilizer_mass.data["mass"].value())
                     << std::endl;
  geom2::Point_3 vs_position_mac = geom2::measure::offset_LE(
      data->vertical_stabilizer, geom2::measure::mean_aerodynamic_chord_position(data->vertical_stabilizer));
  data->vertical_stabilizer_mass.data["x"] =
      data->empennage_position.x.value() + data->vertical_stabilizer.origin.x() + vs_position_mac.x() +
      0.25 * geom2::measure::chord(data->vertical_stabilizer, vs_position_mac.z()) +
      0.1 * geom2::measure::mean_aerodynamic_chord(data->vertical_stabilizer);
  data->vertical_stabilizer_mass.data["y"] = data->vertical_stabilizer.origin.y(); /* Assume symmetric mass */
  data->vertical_stabilizer_mass.data["z"] =
      data->empennage_position.z.value() + data->vertical_stabilizer.origin.z() +
      std::fabs(geom2::measure::mean_aerodynamic_chord_position(data->vertical_stabilizer));

  /* Horizontal stabilizer mass and cog */
  data->horizontal_stabilizer_mass.data["mass"] =
      flops::horizontal_stabilizer_mass(geom2::measure::reference_area(data->horizontal_stabilizer),
                                        geom2::measure::taper_ratio(data->horizontal_stabilizer), maximum_takeoff_mass);
  myRuntimeInfo->out << std::format("Horizontal stabilizer mass ... {:.2f}",
                                    data->horizontal_stabilizer_mass.data["mass"].value())
                     << std::endl;

  /* Cog calculation */
  geom2::Point_3 hs_position_mac = geom2::measure::offset_LE(
      data->horizontal_stabilizer, geom2::measure::mean_aerodynamic_chord_position(data->horizontal_stabilizer));

  data->horizontal_stabilizer_mass.data["x"] =
      data->empennage_position.x.value() + data->horizontal_stabilizer.origin.x() + hs_position_mac.x() +
      0.25 * geom2::measure::chord(data->horizontal_stabilizer, hs_position_mac.z()) +
      0.1 * geom2::measure::mean_aerodynamic_chord(data->vertical_stabilizer);

  data->horizontal_stabilizer_mass.data["y"] = data->horizontal_stabilizer.origin.y(); /* Assume symmetric mass */

  data->horizontal_stabilizer_mass.data["z"] =
      data->empennage_position.z.value() + data->horizontal_stabilizer.origin.z() + -hs_position_mac.z();
  data->empennage_mass.data["mass"] =
      data->horizontal_stabilizer_mass.data["mass"].value() + data->vertical_stabilizer_mass.data["mass"].value();

  /* Compute x direction overall COG_x */
  data->empennage_mass.data["x"] =
      (data->vertical_stabilizer_mass.data["mass"].value() * data->vertical_stabilizer_mass.data["x"].value() +
       data->horizontal_stabilizer_mass.data["mass"].value() * data->horizontal_stabilizer_mass.data["x"].value()) /
      data->empennage_mass.data["mass"].value();

  /* Compute y direction overall COG_y */
  data->empennage_mass.data["y"] =
      (data->vertical_stabilizer_mass.data["mass"].value() * data->vertical_stabilizer_mass.data["y"].value() +
       data->horizontal_stabilizer_mass.data["mass"].value() * data->horizontal_stabilizer_mass.data["y"].value()) /
      data->empennage_mass.data["mass"].value();

  /* Compute z direction overall COG_z */
  data->empennage_mass.data["z"] =
      (data->vertical_stabilizer_mass.data["mass"].value() * data->vertical_stabilizer_mass.data["z"].value() +
       data->horizontal_stabilizer_mass.data["mass"].value() * data->horizontal_stabilizer_mass.data["z"].value()) /
      data->empennage_mass.data["mass"].value();
  myRuntimeInfo->out << std::format("Empennage mass ... {}", data->empennage_mass.data["mass"].value()) << std::endl;
  myRuntimeInfo->out << std::format("Empennage cog  ... {:.2f}, {:.2f}, {:.2f}", data->empennage_mass.data["x"].value(),
                                    data->empennage_mass.data["y"].value(), data->empennage_mass.data["z"].value())
                     << std::endl;
}

void T_Tail::raymer_mass() {
  /* ToDo */
  myRuntimeInfo->err << "Method not implemented yet .... -> no output generated";
}

}  // namespace low
}  // namespace taw
