/**
 * \file cantileverWingDesignReport.cpp
 * \author Christopher Ruwisch (christopher.ruwisch@tu-berlin.de)
 * \brief Report for cantilever wing design
 * \version 0.1
 * \date 2024-02-14
 *
 * @copyright Copyright (c) 2024
 *
 */
#include <aircraftGeometry2/processing/measure.h>
#include <aircraftGeometry2/processing/transform.h>
#include <moduleBasics/html.h>

#include "low_t_tail_empennage_design.h"

namespace taw {
namespace low {
void T_Tail::set_html_body() {
  /* Add box data */
  reporter.htmlReportStream() << "<div class=\"box data\">\n";
  /* Add headline Data*/
  reporter.htmlReportStream() << "<h2>Data</h2>\n";
  /* Table 1 - General parameters */
  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Vertical Stabilizer</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>parameter</th><th>symbol</th><th>unit</th><th>value</th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n"
      << "<tr>\n"
      << std::format("<td>Reference area</td><td>S<sub>ref</sub></td><td>m<sup>2</sup></td><td>{:.2f}</td>\n",
                     geom2::measure::reference_area(data->vertical_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Mean aero. chord</td><td>MAC</td><td>m</td><td>{:.2f}</td>\n",
                     geom2::measure::mean_aerodynamic_chord(data->vertical_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Span</td><td>b</td><td>m</td><td>{:.2f}</td>\n",
                     geom2::measure::span(data->vertical_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Aspect ratio</td><td>AR</td><td>1</td><td>{:.2f}</td>\n",
                     geom2::measure::aspect_ratio(data->vertical_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Taper ratio</td><td>&lambda;</td><td>1</td><td>{:.2f}</td>\n",
                     geom2::measure::taper_ratio(data->vertical_stabilizer))
      << "</tr>\n"
      << std::format("<td>Ref. position</td><td>x<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.x.value() + data->vertical_stabilizer.origin.x())
      << "</tr>\n"
      << std::format("<td></td><td>y<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.y.value() + data->vertical_stabilizer.origin.y())
      << "</tr>\n"
      << std::format("<td></td><td>z<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.z.value() + -data->vertical_stabilizer.origin.z())
      << "</tr>\n"
      << "</tbody>\n"
      << "</table>\n";

  /* Table 2 - Section parameters*/
  reporter.htmlReportStream() << "<table class=\"content-table\">\n"
                              << "<caption>Section Parameter - Vertical Stabilizer</caption>\n"
                              << "<thead>\n"
                              << "<tr>\n"
                              << "<th>parameter</th><th>symbol</th><th>unit</th>";
  for (int i = 0; i < data->vertical_stabilizer.sections.size(); ++i) {
    reporter.htmlReportStream() << std::format("<th>S<sub>{:02d}</sub></th>", i);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "</thead>\n"
                              << "<tbody>\n"
                              /* Dimensionless half span - eta*/
                              << "<tr>\n"
                              << "<td>Dimensionless half span</td><td>&eta;</td><td>1</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>", std::fabs(section.origin.z() / geom2::measure::span(data->vertical_stabilizer)));
  }
  /* Spanwise coordinate - y*/
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "<td>Spanwise coordinate</td><td>y</td><td>m</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", std::fabs(section.origin.z()));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Chord - c*/
                              << "<tr>\n"
                              << "<td>Chord</td><td>c</td><td>m</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", section.get_chord_length());
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - Leading edge - phi_LE*/
                              << "<tr>\n"
                              << "<td>Sweep leading edge</td><td>&phi;<sub>LE</sub></td><td>&deg;</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->vertical_stabilizer, section.origin.z(), 0.0));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - 25 percent - phi_25%*/
                              << "<tr>\n"
                              << "<td>Sweep quarter chord</td><td>&phi;<sub>25</sub></td><td>&deg;</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->vertical_stabilizer, section.origin.z(), 0.25));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - Trailing edge percent - phi_TE*/
                              << "<tr>\n"
                              << "<td>Sweep Trailing edge</td><td>&phi;<sub>TE</sub></td><td>&deg;</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->vertical_stabilizer, section.origin.z(), 1.0));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Dihedral - nu*/
                              << "<tr>\n"
                              << "<td>Dihedral</td><td>&nu;</td><td>&deg;</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        geom2::detail::to_degrees * geom2::measure::dihedral(data->vertical_stabilizer, section.origin.z()));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Twist - epsilon*/
                              << "<tr>\n"
                              << "<td>Twist angle</td><td>&epsilon;</td><td>&deg;</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", geom2::detail::to_degrees * section.rotation_z);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Thickness to chord - epsilon*/
                              << "<tr>\n"
                              << "<td>Thickness ratio</td><td>t/c</td><td>%</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    /* Remove when thickness max is working */
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>", 100 * geom2::measure::thickness_max(section) / section.get_chord_length());
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Airfoil - none */
                              << "<tr>\n"
                              << "<td>Airfoil</td><td>-</td><td>-</td>";
  for (auto section : data->vertical_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{}</td>", section.name);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "</tbody>\n"
                              << "</table>\n";

  /* Table 3 - Spar parameters*/
  std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> vertical_stabilizer_spars =
      data->vertical_stabilizer_spars;
  for (int spar_id = 0; spar_id < vertical_stabilizer_spars.size(); ++spar_id) {
    reporter.htmlReportStream() << "<table class=\"content-table\">\n";
    if (spar_id == 0) {
      reporter.htmlReportStream() << "<caption>Spar parameters</caption>\n";
    }
    reporter.htmlReportStream() << "<thead>\n"
                                << "<tr>\n"
                                << std::format("<th>{:<10}</th><th>&eta; [1]<th>x/c [%]</th>",
                                               vertical_stabilizer_spars.at(spar_id).name)
                                << "\n</tr>\n"
                                << "</thead>\n"
                                << "<tbody>\n";
    /* Spar name - eta, x/c*/
    for (int i = 0; i < vertical_stabilizer_spars.at(spar_id).sections.size(); ++i) {
      reporter.htmlReportStream() << "<tr>\n";
      std::string tag = "";
      if (i == 0) {
        tag = "from";
      } else if (i == vertical_stabilizer_spars.at(spar_id).sections.size() - 1) {
        tag = "to";
      }
      reporter.htmlReportStream() << std::format(
          "<td>{}</td><td>{:.2f}</td><td>{:.2f}</td>\n", tag,
          vertical_stabilizer_spars.at(spar_id).sections.at(i).origin.z(),
          vertical_stabilizer_spars.at(spar_id).sections.at(i).get_contour().vertex(0).x());
      reporter.htmlReportStream() << "\n</tr>\n";
    }
    /* Spanwise coordinate - y*/
    reporter.htmlReportStream() << "</tbody>\n"
                                << "</table>\n";
  }

  /* Sort devices to leading, trailing and other devices*/
  /* device data - name, eta_from, rel_chord_from_start, rel_chord_from_end, eta_to, rel_chord_to_start,
   * rel_chord_to_end */
  using device_data = std::tuple<std::string, double, double, double, double, double, double>;
  std::vector<device_data> vertical_stabilizer_devices;
  std::vector<device_data> horizontal_stabilizer_devices;
  std::vector<device_data> other_devices;
  std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> elevators =
      data->horizontal_stabilizer_control_devices;
  std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> rudders = data->vertical_stabilizer_control_devices;

  for (auto device : rudders) {
    /* Device has only two sections (from + to) spanwise */
    double eta_from = device.sections.at(0).origin.z();
    double rel_chord_from_start = device.sections.at(0).get_contour().vertex(0).x(),
           rel_chord_from_end = device.sections.at(0).get_contour().vertex(1).x();
    double eta_to = device.sections.at(1).origin.z();
    double rel_chord_to_start = device.sections.at(1).get_contour().vertex(0).x(),
           rel_chord_to_end = device.sections.at(1).get_contour().vertex(1).x();
    vertical_stabilizer_devices.push_back({device.name, eta_from, rel_chord_from_start, rel_chord_from_end, eta_to,
                                           rel_chord_to_start, rel_chord_to_end});
  }

  for (auto device : elevators) {
    /* Device has only two sections (from + to) spanwise */
    double eta_from = device.sections.at(0).origin.z();
    double rel_chord_from_start = device.sections.at(0).get_contour().vertex(0).x(),
           rel_chord_from_end = device.sections.at(0).get_contour().vertex(1).x();
    double eta_to = device.sections.at(1).origin.z();
    double rel_chord_to_start = device.sections.at(1).get_contour().vertex(0).x(),
           rel_chord_to_end = device.sections.at(1).get_contour().vertex(1).x();
    horizontal_stabilizer_devices.push_back({device.name, eta_from, rel_chord_from_start, rel_chord_from_end, eta_to,
                                             rel_chord_to_start, rel_chord_to_end});
  }
  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Vertical stabilizer control devices</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>Device</th><th>&eta;<sub>from</sub></th><th>x/c<sub>from,start</sub></th><th>x/c<sub>from,end</sub></"
         "th><th>&eta;<sub>to</sub></th><th>x/c<sub>to,start</sub></th><th>x/c<sub>to,end</sub></th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n";
  for (auto device : vertical_stabilizer_devices) {
    const auto [name, eta_from, rel_chord_from_start, rel_chord_from_end, eta_to, rel_chord_to_start,
                rel_chord_to_end] = device;
    reporter.htmlReportStream() << "<tr>\n";
    reporter.htmlReportStream() << std::format(
        "<td>{}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td>\n", name,
        eta_from, rel_chord_from_start, rel_chord_from_end, eta_to, rel_chord_to_start, rel_chord_to_end);
    reporter.htmlReportStream() << "\n</tr>\n";
  }
  reporter.htmlReportStream() << "</tbody>\n"
                              << "</table>\n";

  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Mass and CoG - Vertical stabilizer</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>Mass</th><th>CoG<sub>x</sub></th><th>CoG<sub>y</sub></th><th>CoG<sub>z</sub></th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n"
      << "<tr>\n"
      << std::format("<td>{:.2f}kg</td><td>{:.2f}m</td><td>{:.2f}m</td><td>{:.2f}m</td>",
                     data->vertical_stabilizer_mass.data["mass"].value(),
                     data->vertical_stabilizer_mass.data["x"].value(), data->vertical_stabilizer_mass.data["y"].value(),
                     data->vertical_stabilizer_mass.data["z"].value())
      << "\n<tr>\n"
      << "</tbody>\n"
      << "</table>\n";

  /* Horizontal stabilizer */
  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Horizontal Stabilizer</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>parameter</th><th>symbol</th><th>unit</th><th>value</th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n"
      << "<tr>\n"
      << std::format("<td>Reference area</td><td>S<sub>ref</sub></td><td>m<sup>2</sup></td><td>{:.2f}</td>\n",
                     geom2::measure::reference_area(data->horizontal_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Mean aero. chord</td><td>MAC</td><td>m</td><td>{:.2f}</td>\n",
                     geom2::measure::mean_aerodynamic_chord(data->horizontal_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Span</td><td>b</td><td>m</td><td>{:.2f}</td>\n",
                     geom2::measure::span(data->horizontal_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Aspect ratio</td><td>AR</td><td>1</td><td>{:.2f}</td>\n",
                     geom2::measure::aspect_ratio(data->horizontal_stabilizer))
      << "</tr>\n"
      << "<tr>\n"
      << std::format("<td>Taper ratio</td><td>&lambda;</td><td>1</td><td>{:.2f}</td>\n",
                     geom2::measure::taper_ratio(data->horizontal_stabilizer))
      << "</tr>\n"
      << std::format("<td>Ref. position</td><td>x<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.x.value() + data->horizontal_stabilizer.origin.x())
      << "</tr>\n"
      << std::format("<td></td><td>y<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.y.value() + data->horizontal_stabilizer.origin.y())
      << "</tr>\n"
      << std::format("<td></td><td>z<sub>ref</sub></td><td>m</td><td>{:.2f}</td>\n",
                     data->empennage_position.z.value() + -data->horizontal_stabilizer.origin.z())
      << "</tr>\n"
      << "</tbody>\n"
      << "</table>\n";

  /* Table 2 - Section parameters*/
  reporter.htmlReportStream() << "<table class=\"content-table\">\n"
                              << "<caption>Section Parameter - Horizontal Stabilizer</caption>\n"
                              << "<thead>\n"
                              << "<tr>\n"
                              << "<th>parameter</th><th>symbol</th><th>unit</th>";
  for (int i = 0; i < data->horizontal_stabilizer.sections.size(); ++i) {
    reporter.htmlReportStream() << std::format("<th>S<sub>{:02d}</sub></th>", i);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "</thead>\n"
                              << "<tbody>\n"
                              /* Dimensionless half span - eta*/
                              << "<tr>\n"
                              << "<td>Dimensionless half span</td><td>&eta;</td><td>1</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>", std::fabs(2 * section.origin.z() / geom2::measure::span(data->horizontal_stabilizer)));
  }
  /* Spanwise coordinate - y*/
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "<td>Spanwise coordinate</td><td>y</td><td>m</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", std::fabs(section.origin.z()));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Chord - c*/
                              << "<tr>\n"
                              << "<td>Chord</td><td>c</td><td>m</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", section.get_chord_length());
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - Leading edge - phi_LE*/
                              << "<tr>\n"
                              << "<td>Sweep leading edge</td><td>&phi;<sub>LE</sub></td><td>&deg;</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->horizontal_stabilizer, section.origin.z(), 0.0));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - 25 percent - phi_25%*/
                              << "<tr>\n"
                              << "<td>Sweep quarter chord</td><td>&phi;<sub>25</sub></td><td>&deg;</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->horizontal_stabilizer, section.origin.z(), 0.25));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Sweep - Trailing edge percent - phi_TE*/
                              << "<tr>\n"
                              << "<td>Sweep Trailing edge</td><td>&phi;<sub>TE</sub></td><td>&deg;</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        -geom2::detail::to_degrees * geom2::measure::sweep(data->horizontal_stabilizer, section.origin.z(), 1.0));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Dihedral - nu*/
                              << "<tr>\n"
                              << "<td>Dihedral</td><td>&nu;</td><td>&deg;</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>",
        geom2::detail::to_degrees * geom2::measure::dihedral(data->horizontal_stabilizer, section.origin.z()));
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Twist - epsilon*/
                              << "<tr>\n"
                              << "<td>Twist angle</td><td>&epsilon;</td><td>&deg;</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{:.2f}</td>", geom2::detail::to_degrees * section.rotation_z);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Thickness to chord - epsilon*/
                              << "<tr>\n"
                              << "<td>Thickness ratio</td><td>t/c</td><td>%</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format(
        "<td>{:.2f}</td>", 100 * geom2::measure::thickness_max(section) / section.get_chord_length());
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              /* Airfoil - none */
                              << "<tr>\n"
                              << "<td>Airfoil</td><td>-</td><td>-</td>";
  for (auto section : data->horizontal_stabilizer.sections) {
    reporter.htmlReportStream() << std::format("<td>{}</td>", section.name);
  }
  reporter.htmlReportStream() << "\n</tr>\n"
                              << "</tbody>\n"
                              << "</table>\n";

  /* Table 3 - Spar parameters*/
  std::vector<geom2::MultisectionSurface<geom2::PolygonSection>> horizontal_stabilizer_spars =
      data->horizontal_stabilizer_spars;
  for (int spar_id = 0; spar_id < horizontal_stabilizer_spars.size(); ++spar_id) {
    reporter.htmlReportStream() << "<table class=\"content-table\">\n";
    if (spar_id == 0) {
      reporter.htmlReportStream() << "<caption>Spar parameters</caption>\n";
    }
    reporter.htmlReportStream() << "<thead>\n"
                                << "<tr>\n"
                                << std::format("<th>{:<10}</th><th>&eta; [1]<th>x/c [%]</th>",
                                               horizontal_stabilizer_spars.at(spar_id).name)
                                << "\n</tr>\n"
                                << "</thead>\n"
                                << "<tbody>\n";
    /* Spar name - eta, x/c*/
    for (int i = 0; i < horizontal_stabilizer_spars.at(spar_id).sections.size(); ++i) {
      reporter.htmlReportStream() << "<tr>\n";
      std::string tag = "";
      if (i == 0) {
        tag = "from";
      } else if (i == horizontal_stabilizer_spars.at(spar_id).sections.size() - 1) {
        tag = "to";
      }
      reporter.htmlReportStream() << std::format(
          "<td>{}</td><td>{:.2f}</td><td>{:.2f}</td>\n", tag,
          horizontal_stabilizer_spars.at(spar_id).sections.at(i).origin.z(),
          horizontal_stabilizer_spars.at(spar_id).sections.at(i).get_contour().vertex(0).x());
      reporter.htmlReportStream() << "\n</tr>\n";
    }
    /* Spanwise coordinate - y*/
    reporter.htmlReportStream() << "</tbody>\n"
                                << "</table>\n";
  }

  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Horizontal stabilizer control devices</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>Device</th><th>&eta;<sub>from</sub></th><th>x/c<sub>from,start</sub></th><th>x/c<sub>from,end</sub></"
         "th><th>&eta;<sub>to</sub></th><th>x/c<sub>to,start</sub></th><th>x/c<sub>to,end</sub></th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n";
  for (auto device : horizontal_stabilizer_devices) {
    const auto [name, eta_from, rel_chord_from_start, rel_chord_from_end, eta_to, rel_chord_to_start,
                rel_chord_to_end] = device;
    reporter.htmlReportStream() << "<tr>\n";
    reporter.htmlReportStream() << std::format(
        "<td>{}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td><td>{:.2f}</td>\n", name,
        eta_from, rel_chord_from_start, rel_chord_from_end, eta_to, rel_chord_to_start, rel_chord_to_end);
    reporter.htmlReportStream() << "\n</tr>\n";
  }
  reporter.htmlReportStream() << "</tbody>\n"
                              << "</table>\n";

  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Mass and CoG - Horizontal stabilizer</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>Mass</th><th>CoG<sub>x</sub></th><th>CoG<sub>y</sub></th><th>CoG<sub>z</sub></th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n"
      << "<tr>\n"
      << std::format(
             "<td>{:.2f}kg</td><td>{:.2f}m</td><td>{:.2f}m</td><td>{:.2f}m</td>",
             data->horizontal_stabilizer_mass.data["mass"].value(), data->horizontal_stabilizer_mass.data["x"].value(),
             data->horizontal_stabilizer_mass.data["y"].value(), data->horizontal_stabilizer_mass.data["z"].value())
      << "\n<tr>\n"
      << "</tbody>\n"
      << "</table>\n";

  reporter.htmlReportStream()
      << "<table class=\"content-table\">\n"
      << "<caption>Mass and CoG - Empennage</caption>\n"
      << "<thead>\n"
      << "<tr>\n"
      << "<th>Mass</th><th>CoG<sub>x</sub></th><th>CoG<sub>y</sub></th><th>CoG<sub>z</sub></th>\n"
      << "</tr>\n"
      << "</thead>\n"
      << "<tbody>\n"
      << "<tr>\n"
      << std::format("<td>{:.2f}kg</td><td>{:.2f}m</td><td>{:.2f}m</td><td>{:.2f}m</td>",
                     data->empennage_mass.data["mass"].value(), data->empennage_mass.data["x"].value(),
                     data->empennage_mass.data["y"].value(), data->empennage_mass.data["z"].value())
      << "\n<tr>\n"
      << "</tbody>\n"
      << "</table>\n";


  /* close box data div */
  reporter.htmlReportStream() << "</div>\n";

  if (rtIO->plotOn) {
    /* open box plot div */
    reporter.htmlReportStream() << "<div class=\"box plot\">\n";
    /* Add headline Data*/
    reporter.htmlReportStream() << "<h2>Plots</h2>\n";

    /* Add Geometry Headline */
    reporter.htmlReportStream() << "<h3>Geometry</h3>\n";


    /* Vertical stabilizer planform plot */
    auto planform_vertical_stabilizer_plot_path = plot_stabilizer_planform(
        data->vertical_stabilizer, data->vertical_stabilizer_spars, data->vertical_stabilizer_control_devices);
    reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n",
                                               planform_vertical_stabilizer_plot_path.string());
    /* Vertical stabilizer airfoil */
    auto vertical_stabilizer_airfoil_plot_paths = plot_airfoils(data->vertical_stabilizer);
    reporter.htmlReportStream() << "<h3>Vertical Stabilizer Airfoils</h3>\n";
    for( auto airfoil_path : vertical_stabilizer_airfoil_plot_paths) {
      reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n", airfoil_path.string());
    }
    reporter.htmlReportStream() << "<h3>Horizontal Stabilizer Thickness & Geometric Twist</h3>\n";
    auto vertical_stabilizer_thickness_and_twist_plot_path = plot_thickness_and_twist_distribution(data->vertical_stabilizer);
    reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n",
    vertical_stabilizer_thickness_and_twist_plot_path.string());


    /* Horizontal Stabilizer planform plot */
    auto planform_horizontal_stabilizer_plot_path = plot_stabilizer_planform(
        data->horizontal_stabilizer, data->horizontal_stabilizer_spars, data->horizontal_stabilizer_control_devices);
    reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n",
                                               planform_horizontal_stabilizer_plot_path.string());

    /* Horizontal stabilizer airfoil */
    auto horizontal_stabilizer_airfoil_plot_paths = plot_airfoils(data->horizontal_stabilizer);
    reporter.htmlReportStream() << "<h3>Horizontal Stabilizer Airfoils</h3>\n";
    for( auto airfoil_path : horizontal_stabilizer_airfoil_plot_paths) {
      reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n", airfoil_path.string());
    }
    reporter.htmlReportStream() << "<h3>Horizontal Stabilizer Thickness & Geometric Twist</h3>\n";
    auto horizontal_stabilizer_thickness_and_twist_plot_path = plot_thickness_and_twist_distribution(data->horizontal_stabilizer);
    reporter.htmlReportStream() << std::format("<img class=\"image-plot\" src=\"{}\"/>\n",
    horizontal_stabilizer_thickness_and_twist_plot_path.string());

    // /* close box plot */
    reporter.htmlReportStream() << "</div>\n";
  }
}
}  // namespace low
}  // namespace taw
